@@ -8,14 +8,12 @@
     (http://www.gnu.org/copyleft/gpl.html)
 """
 
-__version__ = "0.0.3"
+__version__ = "0.0.8"
 
 from .fmm import eikonal
+from .fmm import eikonal_surf
+from .fmm import eikonal_rtp
 
-from eikonal import *
-
-# This is my first C-extension (more will be coming), the first step to make the codes faster
-from calculator import *
 from eikonalc import *
 
 
